@@ -4,8 +4,19 @@ from __future__ import annotations
 
 import argparse
 import dataclasses
-import itertools
-from typing import Any, Callable, Dict, List, Optional, Set, Type, TypeVar, Union, cast
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 
 from typing_extensions import Annotated, get_args, get_origin
 
@@ -31,9 +42,10 @@ class ParserSpecification:
     description: str
     args: List[_arguments.ArgumentDefinition]
     helptext_from_nested_class_field_name: Dict[str, Optional[str]]
-    subparsers_from_name: Dict[str, SubparsersSpecification]
+    subparsers: Optional[SubparsersSpecification]
     prefix: str
     has_required_args: bool
+    consolidate_subcommand_args: bool
 
     @staticmethod
     def from_callable_or_type(
@@ -48,6 +60,12 @@ class ParserSpecification:
     ) -> ParserSpecification:
         """Create a parser definition from a callable or type."""
 
+        # Consolidate subcommand types.
+        consolidate_subcommand_args = (
+            _markers.ConsolidateSubcommandArgs
+            in _resolver.unwrap_annotated(f, _markers.Marker)[1]
+        )
+
         # Resolve generic types.
         f, type_from_typevar = _resolver.resolve_generic_types(f)
         f = _resolver.narrow_type(f, default_instance)
@@ -69,7 +87,7 @@ class ParserSpecification:
         has_required_args = False
         args = []
         helptext_from_nested_class_field_name = {}
-        subparsers_from_name = {}
+        subparsers = None
 
         field_list = _fields.field_list_from_callable(
             f=f, default_instance=default_instance
@@ -111,10 +129,13 @@ class ParserSpecification:
                     ):
                         # Don't make a subparser.
                         field = dataclasses.replace(field, typ=type(field.default))
+                    elif subparsers is None:
+                        subparsers = subparsers_attempt
+                        continue
                     else:
-                        subparsers_from_name[
-                            _strings.make_field_name([prefix, subparsers_attempt.name])
-                        ] = subparsers_attempt
+                        subparsers = add_subparsers_to_leaves(
+                            subparsers, subparsers_attempt
+                        )
                         continue
 
                 # (2) Handle nested callables.
@@ -144,7 +165,14 @@ class ParserSpecification:
                     args.extend(nested_parser.args)
 
                     # Include nested subparsers.
-                    subparsers_from_name.update(nested_parser.subparsers_from_name)
+                    if nested_parser.subparsers is not None:
+                        subparsers = (
+                            nested_parser.subparsers
+                            if subparsers is None
+                            else add_subparsers_to_leaves(
+                                subparsers, nested_parser.subparsers
+                            )
+                        )
 
                     # Include nested strings.
                     for (
@@ -176,15 +204,6 @@ class ParserSpecification:
             if arg.lowered.required:
                 has_required_args = True
 
-        # If a later subparser is required, all previous ones should be as well.
-        subparsers_required = False
-        for name, subparsers in list(subparsers_from_name.items())[::-1]:
-            if subparsers.required:
-                subparsers_required = True
-            subparsers_from_name[name] = dataclasses.replace(
-                subparsers, required=subparsers_required
-            )
-
         return ParserSpecification(
             f=f,
             description=_strings.remove_single_line_breaks(
@@ -194,29 +213,59 @@ class ParserSpecification:
             ),
             args=args,
             helptext_from_nested_class_field_name=helptext_from_nested_class_field_name,
-            subparsers_from_name=subparsers_from_name,
+            subparsers=subparsers,
             prefix=prefix,
             has_required_args=has_required_args,
+            consolidate_subcommand_args=consolidate_subcommand_args,
         )
 
-    def apply(self, parser: argparse.ArgumentParser) -> None:
+    def apply(
+        self, parser: argparse.ArgumentParser
+    ) -> Tuple[argparse.ArgumentParser, ...]:
+        """Create defined arguments and subparsers."""
+
+        # Generate helptext.
+        parser.description = self.description
+
+        # Create subparser tree.
+        if self.subparsers is not None:
+            leaves = self.subparsers.apply(parser)
+        else:
+            leaves = (parser,)
+
+        # Depending on whether we want to consolidate subcommand args, we can either
+        # apply arguments to the intermediate parser or only on the leaves.
+        if self.consolidate_subcommand_args:
+            for leaf in leaves:
+                self.apply_args(leaf)
+        else:
+            self.apply_args(parser)
+
+        # Break some API boundaries to rename the "optional arguments" => "arguments".
+        assert parser._action_groups[1].title == "optional arguments"
+        parser._action_groups[1].title = "arguments"
+
+        return leaves
+
+    def apply_args(self, parser: argparse.ArgumentParser) -> None:
         """Create defined arguments and subparsers."""
 
         # Generate helptext.
         parser.description = self.description
 
         # Make argument groups.
-        def format_group_name(nested_field_name: str) -> str:
-            return (nested_field_name + " arguments").strip()
+        def format_group_name(prefix: str) -> str:
+            return (prefix + " arguments").strip()
 
         group_from_prefix: Dict[str, argparse._ArgumentGroup] = {
             "": parser._action_groups[1],
+            **{
+                cast(str, group.title).partition(" ")[0]: group
+                for group in parser._action_groups[2:]
+            },
         }
-
-        # Break some API boundaries to rename the optional group.
-        parser._action_groups[1].title = format_group_name("")
-        positional_group = parser.add_argument_group("positional arguments")
-        parser._action_groups = parser._action_groups[::-1]
+        positional_group = parser._action_groups[0]
+        assert positional_group.title == "positional arguments"
 
         # Add each argument group. Note that groups with only suppressed arguments won't
         # be added.
@@ -245,14 +294,6 @@ class ParserSpecification:
                 assert arg.lowered.help is argparse.SUPPRESS
                 arg.add_argument(group_from_prefix[""])
 
-        # Create subparser tree.
-        if len(self.subparsers_from_name) > 0:
-            prev_subparser_tree_nodes = [parser]  # Root node.
-            for subparsers in self.subparsers_from_name.values():
-                prev_subparser_tree_nodes = subparsers.apply(
-                    self, prev_subparser_tree_nodes
-                )
-
 
 @dataclasses.dataclass(frozen=True)
 class SubparsersSpecification:
@@ -422,11 +463,9 @@ class SubparsersSpecification:
             default_parser = parser_from_name[default_name]
             if any(map(lambda arg: arg.lowered.required, default_parser.args)):
                 required = True
-            if any(
-                map(
-                    lambda subparsers: subparsers.required,
-                    default_parser.subparsers_from_name.values(),
-                )
+            if (
+                default_parser.subparsers is not None
+                and default_parser.subparsers.required
             ):
                 required = True
 
@@ -460,10 +499,8 @@ class SubparsersSpecification:
         )
 
     def apply(
-        self,
-        parent_parser: ParserSpecification,
-        prev_subparser_tree_nodes: List[argparse.ArgumentParser],
-    ) -> List[argparse.ArgumentParser]:
+        self, parent_parser: argparse.ArgumentParser
+    ) -> Tuple[argparse.ArgumentParser, ...]:
         title = "subcommands"
         metavar = (
             "{"
@@ -481,58 +518,53 @@ class SubparsersSpecification:
             title = "optional " + title
             metavar = f"[{metavar}]"
 
-        subparser_tree_nodes: List[argparse.ArgumentParser] = []
-        for p in prev_subparser_tree_nodes:
-            # Add subparsers to every node in previous level of the tree.
-            argparse_subparsers = p.add_subparsers(
-                dest=_strings.make_subparser_dest(self.prefix),
-                description=self.description,
-                required=self.required,
-                title=title,
-                metavar=metavar,
+        # Add subparsers to every node in previous level of the tree.
+        argparse_subparsers = parent_parser.add_subparsers(
+            dest=_strings.make_subparser_dest(self.prefix),
+            description=self.description,
+            required=self.required,
+            title=title,
+            metavar=metavar,
+        )
+
+        if self.can_be_none:
+            subparser = argparse_subparsers.add_parser(
+                name=_strings.subparser_name_from_type(self.prefix, None),
+                formatter_class=_argparse_formatter.TyroArgparseHelpFormatter,
+                help="",
             )
 
-            if self.can_be_none:
-                subparser = argparse_subparsers.add_parser(
-                    name=_strings.subparser_name_from_type(self.prefix, None),
-                    formatter_class=_argparse_formatter.DcargsArgparseHelpFormatter,
-                    help="",
-                )
-                subparser_tree_nodes.append(subparser)
+        subparser_tree_leaves: List[argparse.ArgumentParser] = []
+        for name, subparser_def in self.parser_from_name.items():
+            helptext = subparser_def.description.replace("%", "%%")
+            if len(helptext) > 0:
+                # TODO: calling a private function here.
+                helptext = _arguments._rich_tag_if_enabled(helptext.strip(), "helptext")
 
-            for name, subparser_def in self.parser_from_name.items():
-                helptext = subparser_def.description.replace("%", "%%")
-                if len(helptext) > 0:
-                    # TODO: calling a private function here.
-                    helptext = _arguments._rich_tag_if_enabled(
-                        helptext.strip(), "helptext"
-                    )
+            subparser = argparse_subparsers.add_parser(
+                name,
+                formatter_class=_argparse_formatter.TyroArgparseHelpFormatter,
+                help=helptext,
+            )
+            subparser_tree_leaves.extend(subparser_def.apply(subparser))
 
-                subparser = argparse_subparsers.add_parser(
-                    name,
-                    formatter_class=_argparse_formatter.DcargsArgparseHelpFormatter,
-                    help=helptext,
-                )
-                subparser_def.apply(subparser)
+        return tuple(subparser_tree_leaves)
 
-                def _get_leaf_subparsers(
-                    node: argparse.ArgumentParser,
-                ) -> List[argparse.ArgumentParser]:
-                    if node._subparsers is None:
-                        return [node]
-                    else:
-                        # Magic!
-                        return list(
-                            itertools.chain(
-                                *map(
-                                    _get_leaf_subparsers,
-                                    node._subparsers._actions[
-                                        -1
-                                    ]._name_parser_map.values(),  # type: ignore
-                                )
-                            )
-                        )
 
-                subparser_tree_nodes.extend(_get_leaf_subparsers(subparser))
+def add_subparsers_to_leaves(
+    root: Optional[SubparsersSpecification], leaf: SubparsersSpecification
+) -> SubparsersSpecification:
+    if root is None:
+        return leaf
 
-        return subparser_tree_nodes
+    new_parsers_from_name = {}
+    for name, parser in root.parser_from_name.items():
+        new_parsers_from_name[name] = dataclasses.replace(
+            parser,
+            subparsers=add_subparsers_to_leaves(parser.subparsers, leaf),
+        )
+    return dataclasses.replace(
+        root,
+        parser_from_name=new_parsers_from_name,
+        required=root.required or leaf.required,
+    )
